package biology;

import utility.SequenceMaker;

public class Sequence {

	public static final int FIVE_TO_THREE = 1;
	public static final int THREE_TO_FIVE = 2;
	public static final int TYPE_DNA = 1;
	public static final int TYPE_RNA = 2;

	private String sequence;
	private int direction;
	private int type;



	public Sequence(String sequence, int direction) {
		this.sequence = sequence.toUpperCase();
		this.direction = direction;
		this.type = TYPE_DNA;
	}
	public Sequence(String sequence, int direction, int type) {
		this.sequence = sequence.toUpperCase();
		this.direction = direction;
		this.type = type;
	}



	public int getType() {
		return type;
	}


	public int getDirection() {
		return direction;
	}

	public String get5To3() {
		if(direction == FIVE_TO_THREE) {
			return sequence.toUpperCase();
		}
		return SequenceMaker.invert(sequence);
	}

	public String get3To5() {
		if(direction == THREE_TO_FIVE) {
			return sequence.toUpperCase();
		}
		return SequenceMaker.invert(sequence);
	}

	public Sequence getComplement() {
		String complementSequence = SequenceMaker.makeComplement(sequence);
		int complementDirection;
		if(direction == FIVE_TO_THREE) {
			complementDirection = THREE_TO_FIVE;
		}
		else {
			complementDirection = FIVE_TO_THREE;
		}

		return new Sequence(complementSequence, complementDirection);
	}

	public String getDNASequence(int wantedDirection) {
		if(direction == wantedDirection) {
			return SequenceMaker.convertRNA2DNA(sequence).toUpperCase();
		}
		else {
			String s = SequenceMaker.invert(sequence);
			return SequenceMaker.convertRNA2DNA(s).toUpperCase();
		}
	}
	public String getRNASequence(int wantedDirection) {
		if(direction == wantedDirection) {
			return SequenceMaker.convertDNA2RNA(sequence).toUpperCase();
		}
		else {
			String s = SequenceMaker.invert(sequence);
			return SequenceMaker.convertDNA2RNA(s).toUpperCase();
		}
	}

	public String toString() {
		return getDNASequence(FIVE_TO_THREE);
	}

	public boolean equals(Sequence sequence) {
		return get3To5().equalsIgnoreCase(sequence.get3To5());
	}

	public boolean equals(Object o) {
		if(o.getClass() == Sequence.class) {
			return equals((Sequence)o);
		}
		return false;
	}

	public int hashCode() {
		return get3To5().hashCode();
	}
}
