package data;

import java.util.LinkedList;

import biology.BioPart;
import biology.BioPartsCollection;
import biology.SequenceCollection;

public class Operation {

	public static final int INPUT = 0;
	public static final int OUTPUT = 1;
	public static final int NOT = 2;
	public static final int AND = 3;
	public static final int OR = 4;

	public static final String[] names = {"INPUT", "OUTPUT", "NOT", "AND", "OR"};


	private int type;
	private boolean value;
	private boolean isValueSet;
	private boolean isOperationSequenceSet;

	private Operation parentOne;
	private Operation parentTwo;

	public int idOfParentBox; /* only public for testing */

	private boolean flag;

	private int inputBioPartID;
	private int outputBioPartID;

	private OperationSequence operationSequence;

	/**
	 * constructor for NOT, AND, OR and OUTPUT
	 * @param type
	 * @param parentOne
	 * @param parentTwo
	 */
	public Operation(int type, Operation parentOne, Operation parentTwo, int idOfParentBox) {
		this.type = type;
		this.parentOne = parentOne;
		this.parentTwo = parentTwo;
		this.isValueSet = false;
		this.operationSequence = new OperationSequence();
		this.idOfParentBox = idOfParentBox;
		outputBioPartID = -1;
		inputBioPartID = -1;
	}

	/**
	 * constructor for INPUT
	 * @param type
	 * @param value
	 * @param parentOne
	 * @param parentTwo
	 */
	public Operation(int type, boolean value, Operation parentOne, Operation parentTwo, int idOfParentBox) {
		this.type = type;
		this.value = value;

		this.parentOne = parentOne;
		this.parentTwo = parentTwo;
		this.isValueSet = true;
		this.operationSequence = new OperationSequence();
		this.idOfParentBox = idOfParentBox;
		outputBioPartID = -1;
		inputBioPartID = -1;
	}




	public int getInputBioPartID() {
		return inputBioPartID;
	}

	public void setInputBioPartID(int inputBioPartID) {
		this.inputBioPartID = inputBioPartID;
	}

	public int getOutputBioPartID() {
		return outputBioPartID;
	}

	public void setOutputBioPartID(int outputBioPartID) {
		this.outputBioPartID = outputBioPartID;
	}

	public boolean isBioPartSet() {
		if(type==AND || type==OR  || type==NOT) {
			return true;
		}
		else if(inputBioPartID > -1 && type==INPUT) {
			return true;
		}
		else if(outputBioPartID > -1 && type==OUTPUT) {
			return true;
		}
		return false;
	}

	public boolean isFlag() {
		return flag;
	}

	public void setFlag(boolean flag) {
		this.flag = flag;
	}

	/**
	 * @return the type
	 */
	public int getType() {
		return type;
	}

	/**
	 * @param type
	 */
	public void setType(int type) {
		this.type = type;
	}



	public Operation getParentOne() {
		return parentOne;
	}

	public void setParentOne(Operation parentOne) {
		this.parentOne = parentOne;
	}

	public Operation getParentTwo() {
		return parentTwo;
	}

	public void setParentTwo(Operation parentTwo) {
		this.parentTwo = parentTwo;
	}

	public boolean addParent(Operation newParent) {
		if (parentOne == null) {
			setParentOne(newParent);
		}
		else if (parentTwo == null) {
			setParentTwo(newParent);
		}
		else {
			return false;
		}

		return true;
	}

	/**
	 * calculate the value if necessary
	 * @return the value
	 */
	public boolean getValue() {

		if(!isValueSet) {
			calculateValue();
		}

		return value;
	}

	public void forceRecalculation() {
		if(type != INPUT) {
			isValueSet = false;
		}
	}
	public void setInputValue(boolean value) {
		if(type == INPUT) {
			this.value = value;
			isValueSet = true;
		}
	}


	/**
	 * calculate value
	 */
	private void calculateValue() {
		switch(type) {

		case OUTPUT:	if(parentOne != null) {
							value = parentOne.getValue();
							isValueSet = true;
							break;
						}

		case NOT:		if(parentOne != null) {
							value = !parentOne.getValue();
							isValueSet = true;
							break;
						}

		case OR:		if(parentOne != null && parentTwo != null) {
							value = parentOne.getValue() || parentTwo.getValue();
							isValueSet = true;
							break;
						}

		case AND:		if(parentOne != null && parentTwo != null) {
							value = parentOne.getValue() && parentTwo.getValue();
							isValueSet = true;
							break;
						}
		}
	}

/*
	public void forceSequenceRecalculation() {
		if(type != INPUT || type != OUTPUT) {
			isOperationSequenceSet = false;
			makeOperationSequence();
		}
	}*/

	public void setIsOperationSequenceSet(boolean flag) {
		isOperationSequenceSet = flag;
	}

	public OperationSequence getOperationSequence() {
		return operationSequence;
	}

	public void makeOperationSequence() {

		operationSequence.empty();

		if(type == INPUT) {
			if(isBioPartSet()) {
				BioPart part = BioPartsCollection.getPart(getInputBioPartID());
				if(part.getType() == BioPart.TYPE_INPUT_PROMOTER) {
					operationSequence.addSubSequence(part.getSequence(), part.getName());
					operationSequence.addSubSequence(SequenceCollection.HIS_SWITCHING, "Trigger site of output transmitter RNA");
					operationSequence.addSubSequence(operationSequence.getOutput(), "Identity site of output transmitter RNA");
					operationSequence.addSubSequence(SequenceCollection.TERMINATOR_SEQUENCE, "Transcriptional terminator");
				}
			}

			isOperationSequenceSet = true;
			return;
		}
		else if(type == OUTPUT) {

			if(isBioPartSet() && parentOne != null) {
				BioPart part = BioPartsCollection.getPart(getOutputBioPartID());
				if(part.getType() == BioPart.TYPE_OUTPUT_BIOBRICK_WITH_RBS || part.getType() == BioPart.TYPE_OUTPUT_CUSTOM_GENE_WITH_RBS || part.getType() == BioPart.TYPE_OUTPUT_CUSTOM_RNA_WITHOUT_RBS) {
					operationSequence.addSubSequence(SequenceCollection.PROMOTOER_SEQUENCE, "Constitutive promoter");
					operationSequence.addSubSequence(parentOne.getOperationSequence().getOutput().getComplement(), "Transmitter recognition site");
					operationSequence.addSubSequence(SequenceCollection.HIS_LOOP_SEQUENCE, "Transcriptional terminator containing the target site");
					operationSequence.addSubSequence(part.getSequence(), part.getName());
					operationSequence.addSubSequence(SequenceCollection.TERMINATOR_SEQUENCE, "Transcriptional terminator");
				}
				else if(part.getType() == BioPart.TYPE_OUTPUT_BIOBRICK_WITHOUT_RBS || part.getType() == BioPart.TYPE_OUTPUT_CUSTOM_PROTEIN_WITHOUT_RBS) {
					operationSequence.addSubSequence(SequenceCollection.PROMOTOER_SEQUENCE, "Constitutive promoter");
					operationSequence.addSubSequence(parentOne.getOperationSequence().getOutput().getComplement(), "Transmitter recognition site");
					operationSequence.addSubSequence(SequenceCollection.HIS_LOOP_SEQUENCE, "Transcriptional terminator containing the target site");
					operationSequence.addSubSequence(SequenceCollection.RBS_SEQUENCE, "Ribosome binding site");
					operationSequence.addSubSequence(part.getSequence(), part.getName());
					operationSequence.addSubSequence(SequenceCollection.TERMINATOR_SEQUENCE, "Transcriptional terminator");
				}
			}

			isOperationSequenceSet = true;
			return;
		}
		else if(type == AND) {

			operationSequence.addSubSequence(SequenceCollection.PROMOTOER_SEQUENCE, "Constitutive promoter");
			operationSequence.addSubSequence(parentOne.getOperationSequence().getOutput().getComplement(), "Transmitter recognition site 1");
			operationSequence.addSubSequence(SequenceCollection.HIS_LOOP_SEQUENCE, "Transcriptional terminator containing the target site");
			operationSequence.addSubSequence(parentTwo.getOperationSequence().getOutput().getComplement(), "Transmitter recognition site 2");
			operationSequence.addSubSequence(SequenceCollection.TRP_LOOP_SEQUENCE, "Transcriptional terminator containing the target site");
			operationSequence.addSubSequence(SequenceCollection.TRP_SWITCHING, "Trigger site of output transmitter RNA");
			operationSequence.addSubSequence(operationSequence.getOutput(), "Identity site of output transmitter RNA");
			operationSequence.addSubSequence(SequenceCollection.TERMINATOR_SEQUENCE, "Transcriptional terminator");

			isOperationSequenceSet = true;
			return;
		}
		else if(type == OR) {

			operationSequence.addSubSequence(SequenceCollection.PROMOTOER_SEQUENCE, "Constitutive promoter");
			operationSequence.addSubSequence(parentOne.getOperationSequence().getOutput().getComplement(), "Transmitter recognition site 1");
			operationSequence.addSubSequence(SequenceCollection.HIS_LOOP_SEQUENCE, "Transcriptional terminator containing the target site");
			operationSequence.addSubSequence(SequenceCollection.HIS_SWITCHING, "Trigger site of output transmitter RNA");
			operationSequence.addSubSequence(operationSequence.getOutput(), "Identity site of output transmitter RNA");
			operationSequence.addSubSequence(SequenceCollection.TERMINATOR_SEQUENCE, "Transcriptional terminator");
			operationSequence.addSubSequence(SequenceCollection.BIOBRICK_SCAR, "Linker");
			operationSequence.addSubSequence(SequenceCollection.PROMOTOER_SEQUENCE, "Constitutive promoter");
			operationSequence.addSubSequence(parentTwo.getOperationSequence().getOutput().getComplement(), "Transmitter recognition site 2");
			operationSequence.addSubSequence(SequenceCollection.HIS_LOOP_SEQUENCE, "Transcriptional terminator containing the target site");
			operationSequence.addSubSequence(SequenceCollection.HIS_SWITCHING, "Trigger site of output transmitter RNA");
			operationSequence.addSubSequence(operationSequence.getOutput(), "Identity site of output transmitter RNA");
			operationSequence.addSubSequence(SequenceCollection.TERMINATOR_SEQUENCE, "Transcriptional terminator");

			isOperationSequenceSet = true;
			return;
		}
		else if(type == NOT) {
			operationSequence.addSubSequence(SequenceCollection.PROMOTOER_SEQUENCE, "Constitutive promoter");
			operationSequence.addSubSequence(SequenceCollection.HIS_SWITCHING, "Internal trigger site of NOT-gate");
			operationSequence.addSubSequence(parentOne.getOperationSequence().getOutput().getComplement(), "Transmitter recognition site");
			operationSequence.addSubSequence(SequenceCollection.HIS_SWITCHING.getComplement(), "Target site of NOT-gate");
			operationSequence.addSubSequence(parentOne.getOperationSequence().getOutput(), "Transmitter identity site");
			operationSequence.addSubSequence(SequenceCollection.HIS_LOOP_SEQUENCE, "Transcriptional terminator containing the target site");
			operationSequence.addSubSequence(SequenceCollection.HIS_SWITCHING, "Trigger site of output transmitter RNA");
			operationSequence.addSubSequence(operationSequence.getOutput(), "Identity site of output transmitter RNA");
			operationSequence.addSubSequence(SequenceCollection.TERMINATOR_SEQUENCE, "Transcriptional terminator");

			isOperationSequenceSet = true;
			return;
		}


		isOperationSequenceSet = false;
	}


	/**
	 * @return true if all necessary parents are defined
	 */
	public boolean checkSanity() {

		switch(type) {

		case INPUT: 	if(parentOne != null || parentTwo != null) {
							return false;
						}
						return true;

		case OUTPUT:	if(parentTwo != null) {
							return false;
						}
						if(parentOne != null) {
							return true;
						}
						return false;

		case NOT:		if(parentTwo != null) {
							return false;
						}
						if(parentOne != null) {
							return true;
						}
						return false;

		case OR:		if(parentOne != null && parentTwo != null) {
							return true;
						}
						return false;

		case AND:		if(parentOne != null && parentTwo != null) {
							return true;
						}
						return false;

		default: 		return false;

		}
	}





	// Funktion gibt eine Liste der Knotenpunkte zurck.
	// Loops werden dadurch signalisiert, dass in der Path-Array der letzte Wert auf -1 gesetzt wird; Gibt es zwei Loops wird der erste detektierte zurckgegeben
	public LinkedList<Integer> getPath(LinkedList<Integer> path)
	{
		boolean newLoop = path.contains(idOfParentBox);
		path.add(idOfParentBox);
		if(newLoop)
		{
			path.add(-1);
			return path;
		}
		if(!newLoop)
		{
			LinkedList<Integer> parentPath = null;
			if(parentOne != null) {
				LinkedList<Integer> newPath = new LinkedList<Integer>();
				newPath.addAll(path);
				parentPath = parentOne.getPath(newPath);
				if(parentPath != null) {
					if(parentPath.size() > 1) {
						if(parentPath.getLast() == -1) {
							return parentPath;
						}
					}
				}
			}
			parentPath = null;
			if(parentTwo != null) {
				LinkedList<Integer> newPath = new LinkedList<Integer>();
				newPath.addAll(path);
				parentPath = parentTwo.getPath(newPath);
				if(parentPath != null) {
					if(parentPath.size() > 1) {
						if(parentPath.getLast() == -1) {
							return parentPath;
						}
					}
				}
			}
		}

		return path;
	}

}