package start;

import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import utility.MyColor;
import utility.SequenceMaker;
import view.CheckPanel;
import view.DNAPanel;
import view.LogicPanel;
import view.Network;
import view.SelectionList;
import biology.BioPartsCollection;

public class MainApplet extends Applet implements ActionListener {

	// for double-buffering
	Graphics bufferGraphics;
	Image offscreen;

	// GUI components
	LogicPanel constructPanel;
	JButton constructButton;

	CheckPanel checkPanel;
	JButton checkButton;

	DNAPanel dnaPanel;
	JButton dnaButton;

	JPanel cards; //a panel that uses CardLayout

	// String are shown in Buttons and are used to identify the different Panels
	public static final String TAB_CONSTRUCT = "1. Construct your own network";
	public static final String TAB_VERIFY = "2. Verify your network";
	public static final String TAB_DNA = "3. Synthesize: Turn your network into DNA";


	//Color backgroundColor;

	@Override
	public void init() {
		super.init();

		// clean up
		Network.clearNetwork();
		BioPartsCollection.initialize();
		SequenceMaker.initializeBlackMap();
		
//		setSize(1000, 700);
//		setPreferredSize(getSize());
		
		offscreen = createImage(this.getSize().width,this.getSize().height);
		bufferGraphics = offscreen.getGraphics();
//		backgroundColor = getBackground();
//		getAndSetParameters();

		constructPanel = new LogicPanel();
		checkPanel = new CheckPanel();
		dnaPanel = new DNAPanel();


        cards = new JPanel(new CardLayout());
        cards.setBorder(BorderFactory.createLineBorder(MyColor.BLACK, 1));
        cards.add(constructPanel, TAB_CONSTRUCT);
        cards.add(checkPanel, TAB_VERIFY);
        cards.add(dnaPanel, TAB_DNA);


		JPanel buttonPanel = new JPanel();

		constructButton = new JButton(TAB_CONSTRUCT);
		constructButton.setActionCommand(TAB_CONSTRUCT);
		constructButton.addActionListener(this);
		buttonPanel.add(constructButton);


		buttonPanel.add(new JLabel("=>"));


		checkButton = new JButton(TAB_VERIFY);
		checkButton.setActionCommand(TAB_VERIFY);
		checkButton.addActionListener(this);
		buttonPanel.add(checkButton);


		buttonPanel.add(new JLabel("=>"));


		dnaButton = new JButton(TAB_DNA);
		dnaButton.setActionCommand(TAB_DNA);
		dnaButton.addActionListener(this);
		buttonPanel.add(dnaButton);


		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout());
		mainPanel.add(cards, BorderLayout.CENTER);
		mainPanel.add(buttonPanel, BorderLayout.NORTH);


		setLayout(new BorderLayout());
		add(mainPanel, BorderLayout.CENTER);
		validate();

		SelectionList.initialize(mainPanel);

		//mainPanel.setBackground(backgroundColor);
				
	}

//	public void getAndSetParameters() {
//		String bgColor = getParameter("bgcolor");
//		if(bgColor != null && bgColor.length() == 6) {
//			int bgRed = Integer.parseInt(bgColor.substring(0, 2), 16);
//			int bgGreen = Integer.parseInt(bgColor.substring(2, 4), 16);
//			int bgBlue = Integer.parseInt(bgColor.substring(4, 6), 16);
//			backgroundColor = new Color(bgRed, bgGreen, bgBlue);
//
//			setBackground(backgroundColor);
//		}
//
//	}



	@Override
	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub

		//Network.forceSequenceRecalculation();

		if(e.getSource().equals(checkButton) && checkPanel != null) {
			List<String> messages = new LinkedList<String>();
			if(!Network.checkForOutput()) {
				messages.add("The network contains no OUTPUTs. Please add at least one OUTPUT.");
			}
			if(Network.hasLoops()) {
				constructPanel.selectLoop();

				messages.add("The network contains at least one LOOP. Please remove all LOOPS.");
			}
			if(Network.hasInsaneBoxes()) {
				constructPanel.selectInsaneBoxes();
				messages.add("The network contains at least one BOX that has not enough / too many connections. Please connect all BOXES with the right number of connections.");
			}

			if(messages.size() > 0) {
				String totalString = "There is at least one error in your network that needs to be removed. To find incorrect elements easier, all erroneous BOXES and CONNECTIONS have been selected.\n\nDetails:";
				for(String s : messages) {
					totalString = totalString + "\n" + s;
				}
				JOptionPane.showMessageDialog(null,
						totalString,
					    "Error found in your network.",
					    JOptionPane.ERROR_MESSAGE);
			}
			else {
				checkPanel.initializeGUI();
				CardLayout cl = (CardLayout)(cards.getLayout());
		        cl.show(cards, (String)e.getActionCommand());
			}

		}

		else if (e.getSource().equals(dnaButton) && dnaPanel != null){
			List<String> messages = new LinkedList<String>();
			if(!Network.checkForOutput()) {
				messages.add("The network contains no OUTPUTs. Please add at least one OUTPUT.");
			}
			if(Network.hasLoops()) {
				constructPanel.selectLoop();

				messages.add("The network contains at least one LOOP. Please remove all LOOPS.");
			}
			if(Network.hasInsaneBoxes()) {
				constructPanel.selectInsaneBoxes();
				messages.add("The network contains at least one BOX that has not enough / too many connections. Please connect all BOXES with the right number of connections.");
			}
			if(Network.hasInputsOrOutputsWithoutBioPart()) {
				constructPanel.selectBoxesWithoutSequence();
				messages.add("The network contains INPUTS or OUTPUTS without sequence.");
			}
			/*
			else if(!Network.checkInputsAndOutputs()) {
				constructPanel.selectBoxesWithNonOrthogonalSequences();
				messages.add("The network contains INPUTS and / or OUTPUTS with sequence that are too similar.");
			}*/
			if(messages.size() > 0) {
				String totalString = "There is at least one error in your network that needs to be removed. To find incorrect elements easier, all erroneous BOXES and CONNECTIONS have been selected.\n\nDetails:";
				for(String s : messages) {
					totalString = totalString + "\n" + s;
				}
				JOptionPane.showMessageDialog(null,
						totalString,
					    "Error found in your network.",
					    JOptionPane.ERROR_MESSAGE);
			}
			else {
				CardLayout cl = (CardLayout)(cards.getLayout());
				cl.show(cards, (String)e.getActionCommand());
				Network.createOperationSequencesForAllBoxes();
				dnaPanel.initializeGUI();
			}
		}

		else {
			CardLayout cl = (CardLayout)(cards.getLayout());
	        cl.show(cards, (String)e.getActionCommand());
		}

	}

	public void paint(Graphics g) {
		//bufferGraphics.setColor(backgroundColor);
		bufferGraphics.fillRect(0,0,this.getSize().width,this.getSize().height);
        super.paint(bufferGraphics);
		g.drawImage(offscreen,0,0,this);
	}
	public void update(Graphics g)
	{
		paint(g);
	}
}
