package utility;

import java.util.HashMap;
import java.util.Map;

import data.Operation;
import data.OperationSequence;

import biology.BioPart;
import biology.BioPartsCollection;
import biology.Sequence;



public class SequenceMaker {


	public static final int STANDARD_LENGTH = 20;
	public static final int STANDARD_DIFFERENCE = 5;

	public static Map<Sequence, Integer> blackMap = new HashMap<Sequence, Integer>();

	public static void initializeBlackMap() {
		blackMap.clear();
		for (BioPart bp : BioPartsCollection.getAllInputParts()) {
			blackMap.put(bp.getSequence(), STANDARD_DIFFERENCE);
		}
		for (BioPart bp : BioPartsCollection.getAllOutputParts()) {
			blackMap.put(bp.getSequence(), STANDARD_DIFFERENCE);
		}
	}
	
	public static void clearBlackMap() {
		blackMap.clear();
	}

	public static Sequence createRNASequence(int direction, int difference, int length) {
		String[] alphabet = {"C","G","A","U"};



		Sequence newSequence;
		String chain;
		do{
			newSequence = null;
			chain = "";
			for(int index = 0; index < length; index++) {
				chain = chain + alphabet[(int)(Math.random() * 4)];
			}

		}while(!checkWithBlacklist(newSequence = new Sequence(chain, direction, Sequence.TYPE_RNA), difference));

		blackMap.put(newSequence, difference);

		return newSequence;
	}

	public static Sequence createDNASequence(int direction, int difference, int length) {
		String[] alphabet = {"C","G","A","T"};

		Sequence newSequence;
		String chain;
		do{
			newSequence = null;
			chain = "";
			for(int index = 0; index < length; index++) {
				chain = chain + alphabet[(int)(Math.random() * 4)];
			}

		}while(!checkWithBlacklist(newSequence = new Sequence(chain, direction, Sequence.TYPE_DNA), difference));

		blackMap.put(newSequence, difference);

		return newSequence;
	}

	private static boolean checkWithBlacklist(Sequence sequence, int value) {

		for(Sequence s : blackMap.keySet()) {
			if(areSimilar(s, sequence, blackMap.get(s))) {
				return false;
			}
		}
		return true;
	}

	/**
	 * @param a
	 * @param b
	 * @param value
	 * @return
	 */
	public static boolean areSimilar(Sequence a, Sequence b, int value) {

		boolean flag = areSimilar(a.get3To5(), b.get3To5(), value);

		Sequence bAsThreeToFive = new Sequence(b.get3To5(), Sequence.FIVE_TO_THREE);

		flag = flag || areSimilar(a.get3To5(), bAsThreeToFive.getComplement().get3To5(), value);

		return flag;

	}

	/**
	 * TODO improve this method
	 *
	 * @param a
	 * @param b
	 * @param value
	 * @return
	 */
	public static boolean areSimilar(String a, String b, int value) { /* WICHTIG: String a aus Blacklist, String b der zu berprfende; value gibt an wie viele Buchstaben anders sein drfen */
	      int requiredSimilarity = a.length() - value;

	      if(b.length() < requiredSimilarity) {
	          return false; /* String der zu berprfen ist, ist zu klein, um genug hnlichkeit zu haben */
	      }
	      if(a.length() < b.length()){ /* String werden so vertauscht, dass String a immer der lngere von beiden ist*/
	          String TEMP = b;
	          b = a;
	          a = TEMP;
	      }
	      for(int j = 0; j <= a.length()-b.length(); j++) {
	          int count = 0;
	          for(int i = 0; i < b.length(); i++) {
	              if(a.charAt(i+j) == b.charAt(i) || Character.toLowerCase(a.charAt(i+j)) == 'n' || Character.toLowerCase(b.charAt(i)) == 'n') { /* erlaubt, dass eine BlacklistSequence ein "N" fr irgendeine Base enthlt */
	                  count++;
	              }
	          }
	          if(count > requiredSimilarity) {
	              return true;
	          }
	      }
	      return false;
	  }

	public static String makeComplement(String sequence) {
		String result = "";
		for(int index = 0; index < sequence.length(); index++) {
			result = result + getComplement(sequence.charAt(index));
		}
		return result;
	}

	public static String invert(String sequence) {
		String result = "";
		for(int index = sequence.length() - 1; index >= 0; index--) {
			result = result + sequence.charAt(index);
		}
		return result;
	}

	private static char getComplement(char base) {
		if(base == 'c' || base == 'C') {
			return 'G';
		}
		if(base == 'g' || base == 'G') {
			return 'C';
		}
		if(base == 'a' || base == 'A') {
			return 'T';
		}
		if(base == 't' || base == 'T' || base == 'u' || base == 'U') {
			return 'A';
		}
		return 'N';
	}

	public static boolean checkRNASequence(String text) {
		for(int index = 0; index < text.length(); index++) {
			char c = text.charAt(index);
			if(c != 'c' && c != 'C' && c != 'g' && c != 'G' && c != 'a' && c != 'A' && c != 'u' && c != 'U' && c != 'n' && c != 'N') {
				return false;
			}
		}
		return true;
	}
	public static boolean checkDNASequence(String text) {
		for(int index = 0; index < text.length(); index++) {
			char c = text.charAt(index);
			if(c != 'c' && c != 'C' && c != 'g' && c != 'G' && c != 'a' && c != 'A' && c != 't' && c != 'T' && c != 'n' && c != 'N') {
				return false;
			}
		}
		return true;
	}


	public static String convertDNA2RNA(String text) {
		text = text.replace('t', 'u');
		text = text.replace('T', 'U');
		return text;
	}
	public static String convertRNA2DNA(String text) {
		text = text.replace('u', 't');
		text = text.replace('U', 'T');
		return text;
	}

	public static void addToBlackMap(Sequence sequence) {
		
		blackMap.put(sequence, SequenceMaker.STANDARD_DIFFERENCE);
		
	}
}
