package view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;

import biology.BioPartsCollection;

import utility.MyColor;

import data.Operation;

public class CheckLeaf extends Leaf implements ActionListener {


	private JLabel label;
	private JLabel infolabel;
	private JCheckBox inputCheckBox;


	/**
	 * constructor
	 */
	public CheckLeaf(Box parentBox) {
		super(parentBox);


		setLayout(new BorderLayout());


		if(parentBox != null && parentBox.getOperation() != null) {
			Operation o = parentBox.getOperation();

			String bioPartsName = "";
			if(o.getInputBioPartID() > -1) {
				bioPartsName = BioPartsCollection.getPart(o.getInputBioPartID()).getName();
			}
			else if(o.getOutputBioPartID() > -1) {
				bioPartsName = BioPartsCollection.getPart(o.getOutputBioPartID()).getName();
			}

			label = new JLabel(Operation.names[parentBox.getOperation().getType()], JLabel.CENTER);
			infolabel = new JLabel(bioPartsName, JLabel.CENTER);
			label.setForeground(MyColor.WHITE);
			infolabel.setForeground(MyColor.WHITE);
			add(label, BorderLayout.NORTH);
			if(infolabel.getText()!=null)
			{
				add(infolabel, BorderLayout.CENTER);
			}

			if(o.getType() == Operation.INPUT) {
				inputCheckBox = new JCheckBox("", o.getValue());
				inputCheckBox.addActionListener(this);
				add(inputCheckBox, BorderLayout.WEST);
			}
		}

		updateBackground();
		validate();
		repaint();
	}





	@Override
	public void setBackground(Color bg) {
		// TODO Auto-generated method stub

		if(label != null) {
			label.setBackground(bg);
		}
		if(inputCheckBox != null) {
			inputCheckBox.setBackground(bg);
		}

		super.setBackground(bg);
	}


	private void updateBackground() {
		if(parentBox != null && parentBox.getOperation() != null) {
			Operation o = parentBox.getOperation();
			if(o.getValue() && getBackground() != MyColor.GREEN) {
				this.setBackground(MyColor.GREEN);
				this.setBorder(BorderFactory.createMatteBorder(1, 1, 1, 1, MyColor.DARK_GREEN));
			}
			else if (!o.getValue() && getBackground() != MyColor.RED){
				this.setBackground(MyColor.RED);
				this.setBorder(BorderFactory.createMatteBorder(1, 1, 1, 1, MyColor.DARK_RED));
			}
		}
	}



	@Override
	public void paint(Graphics arg0) {
		// TODO Auto-generated method stub

		updateBackground();

		super.paint(arg0);
	}





	@Override
	public void actionPerformed(ActionEvent e) {
		// TODO Auto-generated method stub

		if(parentBox != null && parentBox.getOperation() != null && parentBox.getOperation().getType() == Operation.INPUT) {
			Operation o = parentBox.getOperation();
			if(o.getValue() != inputCheckBox.getModel().isSelected())
			{
				o.setInputValue(inputCheckBox.getModel().isSelected());
				Network.forceRecalculation();
				getParent().validate();
				getParent().repaint();
			}
		}
	}
}
