package view;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Label;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import biology.BioPartsCollection;

import utility.MyColor;
import data.Operation;


public class ConstructLeaf extends Leaf implements MouseListener, MouseMotionListener, ActionListener, ItemListener {

	private Point positionBeforeDragging;


	JPanel comboBoxPane;
	private JComboBox typeComboBox;
	private JButton inputButton;
	private JButton outputButton;
	private JLabel inputLabel;
	private JLabel outputLabel;
	JPanel cards;



	/**
	 * constructor
	 */
	public ConstructLeaf(Box parentBox) {
		super(parentBox);

		setLayout(new BorderLayout());
        addMouseMotionListener(this);
        addMouseListener(this);


        comboBoxPane = new JPanel();
        typeComboBox = new JComboBox(Operation.names);
        typeComboBox.setEditable(false);
        typeComboBox.addItemListener(this);
        typeComboBox.setSize(90, 20);
        typeComboBox.setPreferredSize(typeComboBox.getSize());
        typeComboBox.setFont(new Font("Arial", Font.PLAIN, 8));
        typeComboBox.setFocusable(false);
        comboBoxPane.addMouseMotionListener(this);
        comboBoxPane.addMouseListener(this);
        comboBoxPane.add(typeComboBox);

        //Create the "cards".
        JPanel card1 = new JPanel();
        card1.addMouseMotionListener(this);
        card1.addMouseListener(this);
        card1.setLayout(new BorderLayout());
        inputButton = new JButton("Set input...");
        inputButton.addActionListener(this);
        inputButton.setFont(new Font("Arial", Font.PLAIN, 8));
        inputButton.setFocusable(false);
        card1.add(inputButton, BorderLayout.EAST);

        inputLabel = new JLabel();
        if(getParentBox().getOperation().isBioPartSet())
        {
        	inputLabel.setText(BioPartsCollection.getPart(getParentBox().getOperation().getInputBioPartID()).getShortName());
        }
        card1.add(inputLabel, BorderLayout.CENTER);



        JPanel card2 = new JPanel();
        card2.addMouseMotionListener(this);
        card2.addMouseListener(this);
        card2.setLayout(new BorderLayout());
        outputButton = new JButton("Set output...");
        outputButton.addActionListener(this);
        outputButton.setFont(new Font("Arial", Font.PLAIN, 8));
        outputButton.setFocusable(false);
        card2.add(outputButton, BorderLayout.EAST);

        outputLabel = new JLabel();
        if(getParentBox().getOperation().isBioPartSet())
        {
        	outputLabel.setText(BioPartsCollection.getPart(getParentBox().getOperation().getOutputBioPartID()).getShortName());
        }
        card2.add(outputLabel, BorderLayout.CENTER);

        JPanel card3 = new JPanel();
        card3.addMouseMotionListener(this);
        card3.addMouseListener(this);
        JPanel card4 = new JPanel();
        card4.addMouseMotionListener(this);
        card4.addMouseListener(this);
        JPanel card5 = new JPanel();
        card5.addMouseMotionListener(this);
        card5.addMouseListener(this);

        //Create the panel that contains the "cards".
        cards = new JPanel(new CardLayout());
        cards.add(card1, Operation.names[Operation.INPUT]);
        cards.add(card2, Operation.names[Operation.OUTPUT]);
        cards.add(card3, Operation.names[Operation.AND]);
        cards.add(card4, Operation.names[Operation.NOT]);
        cards.add(card5, Operation.names[Operation.OR]);

        add(comboBoxPane, BorderLayout.NORTH);
        add(cards, BorderLayout.CENTER);


        updateBackground();
		validate();
		repaint();
	}




	public void itemStateChanged(ItemEvent evt) {
        CardLayout cl = (CardLayout)(cards.getLayout());
        cl.show(cards, (String)evt.getItem());

		if(parentBox != null && parentBox.getOperation() != null) {
			parentBox.getOperation().setType(typeComboBox.getSelectedIndex());
			parentBox.getOperation().setFlag(false);
			getParent().requestFocus();
		}
    }




	@Override
	public void setBackground(Color bg) {
		// TODO Auto-generated method stub

		if(cards != null)
		{
			for(Component card : cards.getComponents()) {
				card.setBackground(bg);
			}
		}
		if(comboBoxPane != null) {
			comboBoxPane.setBackground(bg);
		}

		super.setBackground(bg);
	}


	private void updateBackground()
	{
		boolean sel = isSelected();
		if(sel && getBackground() != MyColor.LIGHT_BLUE) {
			setBackground(MyColor.LIGHT_BLUE);
			setBorder(BorderFactory.createMatteBorder(1, 1, 1, 1, MyColor.BLUE));
		}
		else if(!sel && getBackground() != MyColor.LIGHT_GRAY) {
			setBackground(MyColor.LIGHT_GRAY);
			setBorder(BorderFactory.createMatteBorder(1, 1, 1, 1, MyColor.GRAY));
		}
	}


	@Override
	public void paint(Graphics g) {
		// TODO Auto-generated method stub

		updateBackground();

		super.paint(g);


		/* just for testing. Whgen delete, make Operation.idOfParentBox private! */
//		if(getParentBox() != null && getParentBox().getOperation() != null)
//		{
//			String s = "";
//			if(getParentBox().getOperation().getParentOne() != null) {
//				s += "Par1 ID="+getParentBox().getOperation().getParentOne().idOfParentBox + " ;";
//			}
//			if(getParentBox().getOperation().getParentTwo() != null) {
//				s += "Par2 ID="+getParentBox().getOperation().getParentTwo().idOfParentBox;
//			}
//			g.setColor(MyColor.BLACK);
//			g.drawString(s, 2, 40);
//		}
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		if(!isSelected()) {
			SelectionList.select(this);
		}
		else {
			SelectionList.deselect(this);
		}
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mousePressed(MouseEvent e) {
		this.positionBeforeDragging = e.getPoint();
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void mouseDragged(MouseEvent e) {
		Point locationLeaf = getLocation();
		Point pointMouse = e.getPoint();

		int x = (int) (locationLeaf.getX() + pointMouse.getX() - positionBeforeDragging.getX());
		int y = (int) (locationLeaf.getY() + pointMouse.getY() - positionBeforeDragging.getY());

		//prevent from leaving viewPanel
		if (x < 0) {
			x = 0;
		}
		if (y < 0) {
			y = 0;
		}
		if(getParent() != null) {
			if (x + this.getSize().width > getParent().getSize().width) {
				x = getParent().getSize().width - this.getSize().width;
			}
			if (y + this.getSize().height > getParent().getSize().height) {
				y = getParent().getSize().height - this.getSize().height;
			}
		}


		parentBox.setLocation(new Point(x,y));
		validate();
		getParent().repaint();
	}

	@Override
	public void mouseMoved(MouseEvent e) {
		// TODO Auto-generated method stub

	}


	@Override
	public void actionPerformed(ActionEvent arg0) {
		// TODO Auto-generated method stub
		if(arg0.getSource().equals(inputButton)) {
			SequenceDialog d = new SequenceDialog(this.getParentBox());
			d.setVisible(true);
			if(BioPartsCollection.getPart(getParentBox().getOperation().getInputBioPartID()) != null)
			{
				inputLabel.setText(BioPartsCollection.getPart(getParentBox().getOperation().getInputBioPartID()).getShortName());
			}
		}
		else if(arg0.getSource().equals(outputButton)) {
			SequenceDialog d = new SequenceDialog(this.getParentBox());
			d.setVisible(true);
			if(BioPartsCollection.getPart(getParentBox().getOperation().getOutputBioPartID()) != null)
			{
				outputLabel.setText(BioPartsCollection.getPart(getParentBox().getOperation().getOutputBioPartID()).getShortName());
			}
		}


		getParent().requestFocus();
	}
}
