package view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

import javax.print.attribute.AttributeSet;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.JTextComponent;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;

import biology.Sequence;
import biology.SequenceCollection;

import utility.SequenceMaker;

import data.Operation;
import data.OperationSequence;

public class DNAPanel extends JPanel {

	JButton plasmidButton;
	JButton elementsButton;
	JButton detailedButton;
	JButton veryDetailedButton;
	StyledDocument doc;
	JTextPane textPane;

	int styleType = -1;


	public void initializeGUI() {
		removeAll();

		setLayout(new BorderLayout());

		textPane = new JTextPane();
		textPane.setEditable(false);
		textPane.setMinimumSize(getSize());
		doc = textPane.getStyledDocument();

		Style def = StyleContext.getDefaultStyleContext().
        getStyle(StyleContext.DEFAULT_STYLE);

		Style regular = doc.addStyle("regular", def);
		StyleConstants.setFontFamily(def, "SansSerif");

		Style s = doc.addStyle("bold and large", regular);
        StyleConstants.setBold(s, true);
        StyleConstants.setFontSize(s, 20);

		s = doc.addStyle("small and underlined", regular);
        StyleConstants.setFontSize(s, 10);
        StyleConstants.setUnderline(s, true);

		s = doc.addStyle("underlined", regular);
        StyleConstants.setUnderline(s, true);

        s = doc.addStyle("red", regular);
        StyleConstants.setForeground(s, Color.RED);

        s = doc.addStyle("blue", regular);
        StyleConstants.setForeground(s, new Color(0, 0, 220));
        s = doc.addStyle("blue and small", regular);
        StyleConstants.setForeground(s, new Color(0, 0, 220));
        StyleConstants.setFontSize(s, 10);

        s = doc.addStyle("orange", regular);
        StyleConstants.setForeground(s, Color.ORANGE);

        s = doc.addStyle("green", regular);
        StyleConstants.setForeground(s, new Color(78, 157, 32));
        s = doc.addStyle("green and small", regular);
        StyleConstants.setForeground(s, new Color(78, 157, 32));
        StyleConstants.setFontSize(s, 10);




		JScrollPane scroller = new JScrollPane(textPane);

		add(scroller, BorderLayout.CENTER);

		JPanel panel = new JPanel();
		panel.setBackground(getBackground());
		panel.setLayout(new FlowLayout());

		plasmidButton = new JButton("Show plasmid");
		plasmidButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				plasmidText();
				textPane.setCaretPosition(0);
			}
		});
		elementsButton = new JButton("Show logical elements");
		elementsButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				elementsOnlyText();
				textPane.setCaretPosition(0);
			}
		});
		detailedButton = new JButton("Show detailed logical elements");
		detailedButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				detailedElementsText();
				textPane.setCaretPosition(0);
			}
		});
		veryDetailedButton = new JButton("Show very detailed logical elements");
		veryDetailedButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				veryDetailedElementsText();
				textPane.setCaretPosition(0);
			}
		});

		panel.add(plasmidButton);
		panel.add(elementsButton);
		panel.add(detailedButton);
		panel.add(veryDetailedButton);
		add(panel, BorderLayout.NORTH);



		// show Elements by default
		elementsOnlyText();
		textPane.setCaretPosition(0);
		
		validate();
		repaint();
	}

	public void plasmidText() {
		textPane.setText("");


		try {
			doc.insertString(doc.getLength(), "Plasmid:" + "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "5'-", doc.getStyle("regular"));
		} catch (BadLocationException e1) {
			e1.printStackTrace();
		}

		for(Box b : Network.getAllInputBoxes()) {
			try {
				doc.insertString(doc.getLength(), b.getOperation().getOperationSequence().getCompleteSequence().toString(), doc.getStyle("regular"));
				doc.insertString(doc.getLength(), SequenceCollection.BIOBRICK_SCAR.toString(), doc.getStyle("regular"));
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}
		for(Box b : Network.getAllSwitchBoxes()) {
			try {
				doc.insertString(doc.getLength(), b.getOperation().getOperationSequence().getCompleteSequence().toString(), doc.getStyle("regular"));
				doc.insertString(doc.getLength(), SequenceCollection.BIOBRICK_SCAR.toString(), doc.getStyle("regular"));
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}
		for(Box b : Network.getAllOutputBoxes()) {
			try {
				doc.insertString(doc.getLength(), b.getOperation().getOperationSequence().getCompleteSequence().toString(), doc.getStyle("regular"));
				doc.insertString(doc.getLength(), SequenceCollection.BIOBRICK_SCAR.toString(), doc.getStyle("regular"));
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}
		try {
			doc.insertString(doc.getLength(), SequenceCollection.BIOBRICK_PLASMID.toString() + "-3'", doc.getStyle("regular"));
		} catch (BadLocationException e1) {
			e1.printStackTrace();
		}


		validate();
		repaint();
	}

	public void elementsOnlyText() {

		textPane.setText("");

		//all inputs
		try {
			doc.insertString(doc.getLength(), "Input Element Sequences" + "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
		} catch (BadLocationException e1) {
			e1.printStackTrace();
		}
		for(Box b : Network.getAllInputBoxes()) {
			try {
				doc.insertString(doc.getLength(), Operation.names[b.getOperation().getType()] + "-gate: \n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "5'-", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), b.getOperation().getOperationSequence().getCompleteSequence().toString(), doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "-3'", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}

		//all switches
		try {
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "Internal Switch Element Sequences" + "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
		} catch (BadLocationException e1) {
			e1.printStackTrace();
		}
		for(Box b : Network.getAllSwitchBoxes()) {
			try {
				
				doc.insertString(doc.getLength(), Operation.names[b.getOperation().getType()] + "-gate: \n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "5'-", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), b.getOperation().getOperationSequence().getCompleteSequence().toString(), doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "-3'", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}

		//all outputs
		try {
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "Output Element Sequences" + "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
		} catch (BadLocationException e1) {
			e1.printStackTrace();
		}
		for(Box b : Network.getAllOutputBoxes()) {
			try {
				doc.insertString(doc.getLength(), Operation.names[b.getOperation().getType()] + "-gate: \n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "5'-", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), b.getOperation().getOperationSequence().getCompleteSequence().toString(), doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "-3'", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}

		validate();
		repaint();
	}

	public void detailedElementsText() {

		textPane.setText("");

		//all inputs
		try {
			doc.insertString(doc.getLength(), "Input Element Sequences" + "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
		} catch (BadLocationException e1) {
			e1.printStackTrace();
		}
		for(Box b : Network.getAllInputBoxes()) {
			try {
				doc.insertString(doc.getLength(), Operation.names[b.getOperation().getType()] + "-gate: \n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "5'-", doc.getStyle("regular"));

				ArrayList<Sequence> subSequences = b.getOperation().getOperationSequence().getSubSequences();
				ArrayList<String> subSequencesName = b.getOperation().getOperationSequence().getNamesOfSubSequence();

				for(int i=0; i<subSequences.size(); i++) {
					doc.insertString(doc.getLength(), " " + subSequences.get(i).toString(), getAlternatingStyle());
					doc.insertString(doc.getLength(), "(" + subSequencesName.get(i) + ")", getAlternatingStyle());
				}
				doc.insertString(doc.getLength(), "-3'", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}

		//all switches
		try {
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "Internal Switch Element Sequences" + "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
		} catch (BadLocationException e1) {
			e1.printStackTrace();
		}
		for(Box b : Network.getAllSwitchBoxes()) {
			try {
				doc.insertString(doc.getLength(), Operation.names[b.getOperation().getType()] + "-gate: \n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "5'-", doc.getStyle("regular"));

				ArrayList<Sequence> subSequences = b.getOperation().getOperationSequence().getSubSequences();
				ArrayList<String> subSequencesName = b.getOperation().getOperationSequence().getNamesOfSubSequence();

				for(int i=0; i<subSequences.size(); i++) {
					doc.insertString(doc.getLength(), " " + subSequences.get(i).toString(), getAlternatingStyle());
					doc.insertString(doc.getLength(), "(" + subSequencesName.get(i) + ")", getAlternatingStyle());
				}
				doc.insertString(doc.getLength(), "-3'", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}

		//all outputs
		try {
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "Output Element Sequences" + "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
		} catch (BadLocationException e1) {
			e1.printStackTrace();
		}
		for(Box b : Network.getAllOutputBoxes()) {
			try {
				doc.insertString(doc.getLength(), Operation.names[b.getOperation().getType()] + "-gate: \n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "5'-", doc.getStyle("regular"));

				ArrayList<Sequence> subSequences = b.getOperation().getOperationSequence().getSubSequences();
				ArrayList<String> subSequencesName = b.getOperation().getOperationSequence().getNamesOfSubSequence();

				for(int i=0; i<subSequences.size(); i++) {
					doc.insertString(doc.getLength(), " " + subSequences.get(i).toString(), getAlternatingStyle());
					doc.insertString(doc.getLength(), "(" + subSequencesName.get(i) + ")", getAlternatingStyle());
				}
				doc.insertString(doc.getLength(), "-3'", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}

		validate();
		repaint();
	}

	public void veryDetailedElementsText() {

		textPane.setText("");

		//all inputs
		try {
			doc.insertString(doc.getLength(), "Input Element Sequences" + "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
		} catch (BadLocationException e1) {
			e1.printStackTrace();
		}
		for(Box b : Network.getAllInputBoxes()) {
			try {
				doc.insertString(doc.getLength(), Operation.names[b.getOperation().getType()] + "-gate: \n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "CompleteSequence: 5'-" + b.getOperation().getOperationSequence().getCompleteSequence().toString() + "-3'", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n\n", doc.getStyle("regular"));
				ArrayList<Sequence> subSequences = b.getOperation().getOperationSequence().getSubSequences();
				ArrayList<String> subSequencesName = b.getOperation().getOperationSequence().getNamesOfSubSequence();
				for(int i=0; i<subSequences.size(); i++) {
					doc.insertString(doc.getLength(), "\n" + subSequencesName.get(i) + ":   ", doc.getStyle("regular"));
					doc.insertString(doc.getLength(), "5'-" + subSequences.get(i).toString() + "-3'", doc.getStyle("regular"));
				}
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}

		//all switches
		try {
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "Internal Switch Element Sequences" + "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
		} catch (BadLocationException e1) {
			e1.printStackTrace();
		}
		for(Box b : Network.getAllSwitchBoxes()) {
			try {
				doc.insertString(doc.getLength(), Operation.names[b.getOperation().getType()] + "-gate: \n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "CompleteSequence: 5'-" + b.getOperation().getOperationSequence().getCompleteSequence().toString() + "-3'", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n\n", doc.getStyle("regular"));
				ArrayList<Sequence> subSequences = b.getOperation().getOperationSequence().getSubSequences();
				ArrayList<String> subSequencesName = b.getOperation().getOperationSequence().getNamesOfSubSequence();
				for(int i=0; i<subSequences.size(); i++) {
					doc.insertString(doc.getLength(), "\n" + subSequencesName.get(i) + ":   ", doc.getStyle("regular"));
					doc.insertString(doc.getLength(), "5'-" + subSequences.get(i).toString() + "-3'", doc.getStyle("regular"));
				}
				doc.insertString(doc.getLength(), "-3'", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}

		//all outputs
		try {
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "Output Element Sequences" + "\n", doc.getStyle("bold and large"));
			doc.insertString(doc.getLength(), "\n", doc.getStyle("bold and large"));
		} catch (BadLocationException e1) {
			e1.printStackTrace();
		}
		for(Box b : Network.getAllOutputBoxes()) {
			try {
				doc.insertString(doc.getLength(), Operation.names[b.getOperation().getType()] + "-gate: \n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "CompleteSequence: 5'-" + b.getOperation().getOperationSequence().getCompleteSequence().toString() + "-3'", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n\n", doc.getStyle("regular"));
				ArrayList<Sequence> subSequences = b.getOperation().getOperationSequence().getSubSequences();
				ArrayList<String> subSequencesName = b.getOperation().getOperationSequence().getNamesOfSubSequence();
				for(int i=0; i<subSequences.size(); i++) {
					doc.insertString(doc.getLength(), "\n" + subSequencesName.get(i) + ":   ", doc.getStyle("regular"));
					doc.insertString(doc.getLength(), "5'-" + subSequences.get(i).toString() + "-3'", doc.getStyle("regular"));
				}
				doc.insertString(doc.getLength(), "-3'", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
				doc.insertString(doc.getLength(), "\n", doc.getStyle("regular"));
			} catch (BadLocationException e) {
				e.printStackTrace();
			}
		}

		validate();
		repaint();
	}

	public Style getAlternatingStyle() {
		styleType++;
		styleType = styleType % 4;
		if(styleType == 0) {
			return doc.getStyle("green");
		}
		else if(styleType == 1) {
			return doc.getStyle("green and small");
		}
		else if(styleType == 2) {
			return doc.getStyle("blue");
		}
		else {
			return doc.getStyle("blue and small");
		}

	}

}
