package view;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.geom.Line2D;


public class LinePlus{

	Box boxFrom;
	Box boxTo;
	Line2D line;
	int id;

	public LinePlus(Box boxFrom, Box boxTo, int id) {
		this.boxFrom = boxFrom;
		this.boxTo = boxTo;
		this.id = id;
	}


	public void paint(Graphics g) {
		if (boxFrom == null || boxTo == null)
		{
			return;
		}


		if(isSelected()){
			g.setColor(Color.BLUE);
		}
		else {
			g.setColor(Color.BLACK);
		}

		Line2D line = getLine();
		drawArrow(g, (int)line.getX1(), (int)line.getY1(), (int)line.getX2(), (int)line.getY2());
	}

	public static void drawArrow(Graphics g, int xFrom, int yFrom, int xTo, int yTo) {
		g.drawLine(xFrom, yFrom, xTo, yTo);
		double beta = 0.3;
		double radius = 20;
		double alpha;
		if(yTo - yFrom >= 0) {
			alpha = Math.atan(((double)xTo-(double)xFrom)/((double)yTo-(double)yFrom)) - Math.PI/2;
		}
		else {
			alpha = Math.atan(((double)xTo-(double)xFrom)/((double)yTo-(double)yFrom)) + Math.PI/2;
		}
		g.fillPolygon(new int[]{xTo, xTo - (int)Math.round( radius * Math.cos(beta-alpha) ) , xTo - (int)Math.round( radius * Math.cos(beta+alpha) )},
				new int[]{yTo, yTo - (int) Math.round( radius * Math.sin(beta-alpha) ), yTo + (int)Math.round( radius * Math.sin(beta+alpha) )}, 3);
	}


	public boolean isSelected() {
		return SelectionList.isSelectedConnector(this);
	}

	public void setSelected(boolean flag) {
		if(flag) {
			SelectionList.select(this);
		}
		else {
			SelectionList.deselect(this);
		}
	}

	public Line2D getLine() {

		int xFrom = boxFrom.getLocation().x;
		int yFrom = boxFrom.getLocation().y;
		int xTo = boxTo.getLocation().x;
		int yTo = boxTo.getLocation().y;

		if( boxFrom.getConstructLeaf() != null && boxFrom.getConstructLeaf() != null ) {
			int widthFrom = boxFrom.getConstructLeaf().getSize().width;
			int widthTo = boxTo.getConstructLeaf().getSize().width;
			int heightFrom = boxFrom.getConstructLeaf().getSize().height;
			int heightTo = boxTo.getConstructLeaf().getSize().height;

			int xArrowStart = xFrom;
			int xArrowEnd = xTo;
			int yArrowStart = yFrom;
			int yArrowEnd = yTo;



			/* y-Koordinate erst geprft, dann x
			if(yTo + heightTo < yFrom) {
				xArrowStart = xFrom + widthFrom/2;
				yArrowStart = yFrom;
				xArrowEnd = xTo + widthTo/2;
				yArrowEnd = yTo + heightTo;
			}
			else if (yTo + heightTo < yFrom + 2 * heightFrom) {
				if(xTo + widthTo < xFrom) {
					xArrowStart = xFrom;
					yArrowStart = yFrom + heightFrom/2;
					xArrowEnd = xTo + widthFrom;
					yArrowEnd = yTo + heightTo/2;
				}
				else {
					xArrowStart = xFrom + widthFrom;
					yArrowStart = yFrom + heightFrom/2;
					xArrowEnd = xTo;
					yArrowEnd = yTo + heightTo/2;
				}
			}
			else {
				xArrowStart = xFrom + widthFrom/2;
				yArrowStart = yFrom + heightFrom;
				xArrowEnd = xTo + widthTo/2;
				yArrowEnd = yTo;
			}
			drawArrow(g, xArrowStart, yArrowStart, xArrowEnd, yArrowEnd);
			*/

			if(xTo + widthTo < xFrom) {
				xArrowStart = xFrom;
				yArrowStart = yFrom + heightFrom/2;
				xArrowEnd = xTo + widthFrom;
				yArrowEnd = yTo + heightTo/2;
			}
			else if(xTo < xFrom + widthFrom) {
				if(yTo + heightTo < yFrom) {
					xArrowStart = xFrom + widthFrom/2;
					yArrowStart = yFrom;
					xArrowEnd = xTo + widthTo/2;
					yArrowEnd = yTo + heightTo;
				}
				else {
					xArrowStart = xFrom + widthFrom/2;
					yArrowStart = yFrom + heightFrom;
					xArrowEnd = xTo + widthTo/2;
					yArrowEnd = yTo;
				}
			}
			else {
				xArrowStart = xFrom + widthFrom;
				yArrowStart = yFrom + heightFrom/2;
				xArrowEnd = xTo;
				yArrowEnd = yTo + heightTo/2;
			}
			return new Line2D.Double(xArrowStart, yArrowStart, xArrowEnd, yArrowEnd);

		}
		else {
			return new Line2D.Double(xFrom, yFrom, xTo, yTo);
		}
	}

	public int getID() {
		return id;
	}

	public Box getBoxFrom() {
		return boxFrom;
	}

	public Box getBoxTo() {
		return boxTo;
	}


}