package view;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JLayeredPane;


public class LogicPanel extends JLayeredPane implements MouseListener, MouseMotionListener, KeyListener {


    /**
     * constructor
     */
    public LogicPanel() {
        addKeyListener(this);
        addMouseListener(this);
        addMouseMotionListener(this);
        setFocusable(true);

        setSize(500, 500);
        setPreferredSize(getSize());



        setLayout(null);
    }



    private Box createNewBox(Point location)
    {
        Box newBox = Network.newBox();
        ConstructLeaf constructLeaf = new ConstructLeaf(newBox);
        add(constructLeaf, new Integer(1));
        newBox.setConstructLeaf(constructLeaf);
        Point newLocation = location;
        if (newLocation.x < 0) {
        	newLocation.x = 0;
        }
        else if (newLocation.x + newBox.getWidth() > this.getSize().width) {
        	newLocation.x = this.getSize().width - newBox.getWidth();
        }
        if (newLocation.y < 0) {
        	newLocation.y = 0;
        }
        else if (newLocation.y + newBox.getHeight() > this.getSize().height) {
        	newLocation.y = this.getSize().height - newBox.getHeight();
        }

        newBox.setLocation(location);
        repaint();
        return newBox;
    }
    private LinePlus createNewConnector(Box boxFrom, Box boxTo)
    {
        LinePlus newConnector = Network.newConnector(boxFrom, boxTo);
        repaint();
        return newConnector;
    }

    private void removeLeaves(LinkedList<Leaf> leaves) {

        for (Leaf l : leaves) {
            Network.remove(l);
            if(l.getParent() == this) {
                remove(l);
            }
        }
    }
    private void removeConnectors(LinkedList<LinePlus> connectors) {

        for(LinePlus l : connectors) {
            Network.remove(l);
        }
    }

    @Override
    public void mouseClicked(MouseEvent e) {
        requestFocusInWindow();

        for(Box b : Network.getAllBoxes()) {
            if(b.getCheckLeaf() != null) {
                SelectionList.deselect(b.getCheckLeaf());
            }
            if(b.getConstructLeaf() != null) {
                SelectionList.deselect(b.getConstructLeaf());
            }
        }

        boolean hasConnectorBeenClicked = false;
        for(LinePlus l : Network.getAllConnectors()) {
            if(l.getLine().ptSegDist(e.getPoint()) < 5) {
                if(!l.isSelected()) {
                    SelectionList.select(l);
                }
                else {
                    SelectionList.deselect(l);
                }
                hasConnectorBeenClicked = true;
            }
        }
        if(!hasConnectorBeenClicked)
        {
            for(LinePlus l : Network.getAllConnectors()) {
                l.setSelected(false);
            }
        }

        if (e.getClickCount() == 2) {
            createNewBox(e.getPoint());
            return;
        }

        validate();
        repaint();
    }

    @Override
    public void mouseEntered(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseExited(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mousePressed(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseReleased(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseDragged(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void mouseMoved(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void keyPressed(KeyEvent e) {
        if(e.getKeyCode() == KeyEvent.VK_BACK_SPACE || e.getKeyCode() == KeyEvent.VK_DELETE) {

            SelectionList.holdUpdates();
            if(SelectionList.getSelectedCollectorsCount() > 0) {
                removeConnectors(SelectionList.getSelectedCollectors());
            }
            if(SelectionList.getSelectedLeavesCount() > 0) {
                removeLeaves(SelectionList.getSelectedLeaves());
            }

            SelectionList.clearSelection();
            SelectionList.startUpdates();
        }
        else if(e.getKeyCode() == KeyEvent.VK_C) {
            if(SelectionList.getSelectedLeaves().size() == 2) {
                if(SelectionList.getSelectedLeaves().get(0).getParentBox() != null && SelectionList.getSelectedLeaves().get(1).getParentBox() != null)
                    createNewConnector(SelectionList.getSelectedLeaves().get(0).getParentBox(), SelectionList.getSelectedLeaves().get(1).getParentBox());
                SelectionList.clearSelection();
            }
        }
        else if(e.getKeyCode() == KeyEvent.VK_D) { /* just for testing */
            repaint();
        }
        else if(e.getKeyCode() == KeyEvent.VK_V) { /* just for testing */
            validate();
        }
    }

    @Override
    public void keyReleased(KeyEvent e) {
        // TODO Auto-generated method stub

    }

    @Override
    public void keyTyped(KeyEvent e) {
        // TODO Auto-generated method stub

    }


    public void paint(Graphics g) {
        for(LinePlus linePlus : Network.getAllConnectors()) {
            linePlus.paint(g);
        }


        String s = "";

        if(Network.getAllBoxes().size() < 1) {
            s="Double-click here to add logic elements";
        }
        else if (Network.getAllConnectors().size() < 1) {
            s="Select two logic elements and press \"C\" to connect these elements";
        }

        if(s != "") {
            g.setFont(new Font("Arial", Font.BOLD, 20));
            g.setColor(new Color(Math.max(getBackground().getRed()-30, 0), Math.max(getBackground().getRed()-30, 0), Math.max(getBackground().getRed()-30, 0)));
            int textWidth = g.getFontMetrics().stringWidth(s);
            g.drawString(s, getSize().width/2 - textWidth/2, getSize().height/2 - g.getFontMetrics().getHeight());
        }

        super.paint(g);
    }

    public void selectLoop() {
        LinkedList<Integer> loop = Network.getLoop();
        if(loop != null && loop.size() > 2) {
            SelectionList.clearSelection();
            int indexDouble = loop.get(loop.size() - 2);
            LinkedList<Integer> loopIndices = new LinkedList<Integer>();
            loopIndices.add(indexDouble);
            for (int i = loop.size() - 3; i > -1; i--) {
                loopIndices.add(loop.get(i));
                if(loop.get(i) == indexDouble) {
                    break;
                }
            }

            /* find and select the connecting loops */
            SelectionList.holdUpdates();
            LinkedList<LinePlus> loopLines = new LinkedList<LinePlus>();

            for(int i=0; i<loopIndices.size() - 1; i++) {
                LinePlus line = Network.getConnectorBetweenBoxes(loopIndices.get(i), loopIndices.get(i+1));
                if(line != null) {
                    loopLines.add(line);
                }
            }

            for(LinePlus line : loopLines) {
                SelectionList.select(line);
            }

            /* Selected leafes within the loop
            for(int i : loopIndices) {
                Box box = network.getBoxByID(i);
                if(box.getConstructLeaf() != null) {
                    SelectionList.select(box.getConstructLeaf());
                }
            }
            */

            SelectionList.startUpdates();
        }
    }

    public void selectInsaneBoxes() {
    	List<Box> insaneBoxes = Network.getInsaneBoxes();

    	SelectionList.holdUpdates();

    	for(Box box : insaneBoxes) {
    		SelectionList.select(box.getConstructLeaf());
    	}

    	SelectionList.startUpdates();
    }

    public void selectBoxesWithoutSequence() {

    	List<Box> boxes = Network.getInputsAndOutputsWithoutBioPart();

    	SelectionList.holdUpdates();

    	for(Box box : boxes) {
    		SelectionList.select(box.getConstructLeaf());
    	}

    	SelectionList.startUpdates();
    }

    /*
    public void selectBoxesWithNonOrthogonalSequences() {

    	Collection<Box> boxes = Network.getInputsAndOutputsWithNonOrthogonalSequences();

    	SelectionList.holdUpdates();

    	for(Box box : boxes) {
    		SelectionList.select(box.getConstructLeaf());
    	}

    	SelectionList.startUpdates();
    }*/
}
