package view;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import utility.SequenceMaker;

import data.Operation;



public class Network {
	private static Collection<LinePlus> connectors = new LinkedList<LinePlus>();
	private static Collection<Box> boxes = new LinkedList<Box>();


	private static int maxBoxId = 0;
	private static int maxConnectorId = 0;

	public static void clearNetwork() {
		connectors.clear();
		boxes.clear();
	}

	public static Collection<LinePlus> getAllConnectors() {
		return connectors;
	}

	public static Collection<Box> getAllBoxes() {
		return boxes;
	}
	public static LinePlus getConnectorByID(int index) {
		for (LinePlus line : getAllConnectors()) {
			if(line.getID()==index) {
				return line;
			}
		}
		return null;
	}
	public static Box getBoxByID( int index) {
		for (Box box : getAllBoxes()) {
			if(box.getID()==index) {
				return box;
			}
		}
		return null;
	}

	public static Box newBox()
	{
		Box newBox = new Box(maxBoxId);
		maxBoxId++;
		boxes.add(newBox);
		newBox.setOperation( new data.Operation(Operation.INPUT, null, null, newBox.getID()) );

		return newBox;
	}
	public static LinePlus newConnector(Box boxFrom, Box boxTo)
	{
		for (LinePlus line : getAllConnectors()) {
			if(line.getBoxFrom().equals(boxFrom) && line.getBoxTo().equals(boxTo)) {
				return null;
			}
		}

		
		if (boxFrom.getOperation() != null && boxTo.getOperation() != null) {
		
			if(!boxTo.getOperation().addParent(boxFrom.getOperation())) {
				return null;
			}
			else {
				LinePlus newConnector = new LinePlus(boxFrom, boxTo, maxConnectorId);
				maxConnectorId++;
				connectors.add(newConnector);
				return newConnector;
			}
		}
		return null;
	}

	public static void remove(Object object)
	{
		if (object instanceof LinePlus) {
			removeConnector( (LinePlus)object );
		}
		else if (object instanceof Leaf) {
			removeBox( ((Leaf)object).getParentBox() );
		}
		else if (object instanceof Box) {
			removeBox( (Box)object );
		}
	}

	public static void removeBox(int boxID) {
		for(Box b : getAllBoxes()) {
			if(b.getID() == boxID) {
				boxes.remove(b);
			}
		}
	}

	public static void forceRecalculation() {
		for(Box b : getAllBoxes()) {
			if(b.getOperation() != null) {
				b.getOperation().forceRecalculation();
			}
		}
	}

/*
	public static void forceSequenceRecalculation() {
		for(Box b : getAllBoxes()) {
			if(b.getOperation() != null) {
				b.getOperation().forceSequenceRecalculation();
			}
		}
	}
*/
	public static void createOperationSequencesForAllBoxes() {

		for(Box b : getAllBoxes()) {
			if(b.getOperation() != null) {
				b.getOperation().makeOperationSequence();
			}
		}
	}

	public static void removeBox(Box box) {
		if(box.getOperation()!=null) {
			for(Box b : getAllBoxes()) {
				if(b.getOperation() != null) {
					if(b.getOperation().getParentOne() == box.getOperation()) {
						b.getOperation().setParentOne(null);
					}
					if(b.getOperation().getParentTwo() == box.getOperation()) {
						b.getOperation().setParentTwo(null);
					}
				}
			}
		}

		Collection<LinePlus> allCollectors = new LinkedList<LinePlus>();
		allCollectors.addAll(Network.getAllConnectors());

		for(LinePlus line : allCollectors) {
			if(line.getBoxFrom() == box || line.getBoxTo() == box)
			{
				Network.remove(line);
			}
		}

		boxes.remove(box);
	}

	public static void removeConnector(int connectorID) {
		for(LinePlus l : getAllConnectors()) {
			if(l.getID() == connectorID) {
				removeConnector(l);
			}
		}

	}
	public static void removeConnector(LinePlus connector) {

		Box bTo = connector.getBoxTo();
		Box bFrom = connector.getBoxFrom();
		if(bTo != null && bFrom != null)
		{
			if(bTo.getOperation() != null) {
				if(bTo.getOperation().getParentOne() == bFrom.getOperation()) { /* remove ParentOne and move parentTwo to parentOne */
					bTo.getOperation().setParentOne(bTo.getOperation().getParentTwo());
					bTo.getOperation().setParentTwo(null);
				}
				if(bTo.getOperation().getParentTwo() == bFrom.getOperation()) {
					bTo.getOperation().setParentTwo(null);
				}
			}
		}

		connectors.remove(connector);
	}

	public static boolean hasLoops() {
		if(getLoop() == null) {
			return false;
		}
		else {
			return true;
		}
	}
	public static LinkedList<Integer> getLoop() {
		for(Box box : getAllBoxes()) {
			if(box.getOperation() != null) {
				if(box.getOperation().getType() == Operation.OUTPUT) {
					LinkedList<Integer> loop = box.getOperation().getPath(new LinkedList<Integer>());
					if(loop.getLast() == -1) {
						return loop;
					}
					else {
						return null;
					}
				}
			}
		}
		return null;
	}

	public static LinePlus getConnectorBetweenBoxes(int fromID, int toID) {
		for (LinePlus line : getAllConnectors()) {
			if(line.getBoxFrom() != null && line.getBoxTo() != null) {
				if(line.getBoxFrom().getID() == fromID && line.getBoxTo().getID() == toID) {
					return line;
				}
			}
		}
		return null;
	}

	public static List<Box> getInsaneBoxes() {
		List<Box> insaneBoxes = new LinkedList<Box>();

		for(Box box : getAllBoxes()) {
			if(!box.getOperation().checkSanity()) {
				insaneBoxes.add(box);
			}
		}

		return insaneBoxes;
	}

	public static List<Box> getInputsAndOutputsWithoutBioPart() {
		List<Box> result = new LinkedList<Box>();

		for(Box box : getAllBoxes()) {
			if(!box.getOperation().isBioPartSet()) {
				result.add(box);
			}
		}

		return result;
	}

/*
	public static List<Box> getInputsAndOutputsWithNonOrthogonalSequences() {

		List<Box> result = new LinkedList<Box>();

		List<Box> tmp = new ArrayList<Box>();

		for(Box box : getAllBoxes()) {
			if(box.getOperation().getType() == Operation.INPUT || box.getOperation().getType() == Operation.OUTPUT) {
				tmp.add(box);
			}
		}

		for(int i = 0; i < tmp.size() - 1; i++) {
			for(int j = i + 1; j < tmp.size(); j++) {

				if(SequenceMaker.areSimilar(tmp.get(i).getOperation().getOutputSequence(), tmp.get(j).getOperation().getOutputSequence(), SequenceMaker.STANDARD_DIFFERENCE)) {
					result.add(tmp.get(i));
					result.add(tmp.get(j));
				}
			}
		}
		return result;
	}*/

	public static boolean checkForOutput() {
		for(Box box : getAllBoxes()) {
			if(box.getOperation().getType() == Operation.OUTPUT) {
				return true;
			}
		}
		return false;
	}

	public static boolean hasInsaneBoxes() {
		if(getInsaneBoxes().size() == 0) {
			return false;
		}
		return true;
	}

	public static boolean hasInputsOrOutputsWithoutBioPart() {
		if(getInputsAndOutputsWithoutBioPart().size() != 0) {
			return true;
		}
		return false;
	}
/*
	public static boolean checkInputsAndOutputs() {
		if(getInputsAndOutputsWithNonOrthogonalSequences().size() > 0) {
			return false;
		}
		return true;
	}*/

	public static LinkedList<Box> getAllInputBoxes()
	{
		LinkedList<Box> result = new LinkedList<Box>();
		for(Box b : getAllBoxes()) {
			if(b.getOperation().getType()== Operation.INPUT) {
				result.add(b);
			}
		}
		return result;
	}
	public static LinkedList<Box> getAllOutputBoxes()
	{
		LinkedList<Box> result = new LinkedList<Box>();
		for(Box b : getAllBoxes()) {
			if(b.getOperation().getType()== Operation.OUTPUT) {
				result.add(b);
			}
		}
		return result;
	}
	public static LinkedList<Box> getAllSwitchBoxes()
	{
		LinkedList<Box> result = new LinkedList<Box>();
		for(Box b : getAllBoxes()) {
			if(b.getOperation().getType()!= Operation.OUTPUT && b.getOperation().getType()!= Operation.INPUT) {
				result.add(b);
			}
		}
		return result;
	}
}
