package view;

import java.util.LinkedList;

import javax.swing.JComponent;



public class SelectionList {
	private static LinkedList<Leaf> selectedLeaves = new LinkedList<Leaf>();
	private static LinkedList<LinePlus> selectedCollectors = new LinkedList<LinePlus>();

	private static JComponent componentToUpdate = null;
	private static boolean doUpdates = false;

	public static void initialize(JComponent comp) {
		selectedLeaves.clear();
		selectedCollectors.clear();
		componentToUpdate = comp;
		doUpdates = true;
	}

	public static void holdUpdates() {
		doUpdates = false;
	}
	public static void startUpdates() {
		doUpdates = true;
		update();
	}
	private static void update() {
		if(doUpdates && componentToUpdate != null) {
			componentToUpdate.revalidate();
			componentToUpdate.repaint();
		}
	}

	public static void clearSelection() {
		selectedLeaves.clear();
		selectedCollectors.clear();
		update();
	}
	public static void clearLeafSelection() {
		selectedLeaves.clear();
		update();
	}
	public static void clearConnectorSelection() {
		selectedCollectors.clear();
		update();
	}

	public static LinkedList<Leaf> getSelectedLeaves() {
		return selectedLeaves;
	}
	public static LinkedList<LinePlus> getSelectedCollectors() {
		return selectedCollectors;
	}
	public static int getSelectedLeavesCount() {
		return selectedLeaves.size();
	}
	public static int getSelectedCollectorsCount() {
		return selectedCollectors.size();
	}
	public static int getTotalSelectedCount() {
		return (selectedLeaves.size() + selectedCollectors.size());
	}

	public static void select(Object o) {
		if(o instanceof Leaf) {
			selectLeaf((Leaf)o);
		}
		else if(o instanceof LinePlus) {
			selectConnector((LinePlus)o);
		}
	}
	public static void selectLeaf(Leaf leaf) {
		if(!selectedLeaves.contains(leaf)) {
			selectedLeaves.add(leaf);
			update();
		}
	}
	public static void selectConnector(LinePlus connector) {
		if(!selectedCollectors.contains(connector)) {
			selectedCollectors.add(connector);
			update();
		}
	}

	public static void deselect(Object o) {
		if(o instanceof Leaf) {
			deselectLeaf((Leaf)o);
		}
		else if(o instanceof LinePlus) {
			deselectConnector((LinePlus)o);
		}
	}
	public static void deselectLeaf(Leaf leaf) {
		if(selectedLeaves.contains(leaf)) {
			selectedLeaves.remove(leaf);
			update();
		}
	}
	public static void deselectConnector(LinePlus connector) {
		if(selectedCollectors.contains(connector)) {
			selectedCollectors.remove(connector);
			update();
		}
	}

	public static boolean isSelectedLeaf(Leaf leaf) {
		return selectedLeaves.contains(leaf);
	}
	public static boolean isSelectedConnector(LinePlus connector) {
		return selectedCollectors.contains(connector);
	}
	public static boolean isSelected(Object o) {
		if(o instanceof Leaf) {
			return isSelectedLeaf((Leaf)o);
		}
		else if(o instanceof LinePlus) {
			return isSelectedConnector((LinePlus)o);
		}
		return false;
	}
}
