package view;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;

import utility.SequenceMaker;
import biology.BioPart;
import biology.BioPartsCollection;
import biology.Sequence;
import data.Operation;

public class SequenceDialog extends JDialog implements ActionListener {

	private JPanel cardPanel;
	private JPanel comboBoxPanel;
	protected JComboBox comboBox;
	protected JComboBox proteinBox;
	protected JPanel sequencePanel;
	protected JPanel infoPanel;
	protected JTextPane instructionTextPane;
	protected JTextPane sequenceTextField;
	protected JTextField nameTextField;
	protected JTextPane infoTextPane;
	protected JScrollPane scroller;
	protected JButton setButton;

	private ArrayList<BioPart> bioParts; /* contains all BiiParts, that can be choosen from */
	String[] selection; /* contains all BioParts-Name, that can be choosen from */

	Box parentBox;

	public SequenceDialog (Box box) {
		parentBox = box;
		int type = parentBox.getOperation().getType();

		setLayout(new BorderLayout());
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setLocationRelativeTo(null);
		setResizable(true);
		setModal(true);
		setSize(300,200);
		setMinimumSize(new Dimension(500,300));


		setTitle("Choose your " + (type==Operation.INPUT ? "input" : "output") + ".");


		comboBoxPanel = new JPanel();
		comboBoxPanel.setLayout(new BorderLayout());


		if(type == Operation.INPUT) {
			setTitle("Choose your input");
			bioParts = new ArrayList<BioPart>(BioPartsCollection.getAllInputParts());
			comboBoxPanel.add(new JLabel("Choose input:"), BorderLayout.WEST);

			selection = new String[bioParts.size()];
		}
		else if(type == Operation.OUTPUT) {
			setTitle("Choose your output");
			bioParts = new ArrayList<BioPart>(BioPartsCollection.getAllOutputParts());
			comboBoxPanel.add(new JLabel("Choose output:"), BorderLayout.WEST);

			selection = new String[bioParts.size() + 3];
			selection[bioParts.size()] = "New custom Gene with RBS";
			selection[bioParts.size()+1] = "New custom Gene without RBS";
			selection[bioParts.size()+2] = "New custom RNA";
		}

		int i=0;
		for(BioPart b : bioParts) {
			selection[i] = b.getName();
			i++;
		}


		comboBox = new JComboBox(selection);
		comboBox.addActionListener(this);
		comboBoxPanel.add(comboBox, BorderLayout.CENTER);

		instructionTextPane = new JTextPane();
		instructionTextPane.setBackground(getBackground());
		comboBoxPanel.add(instructionTextPane, BorderLayout.SOUTH);

		sequencePanel = new JPanel();
		sequencePanel.setLayout(new BorderLayout());
		sequenceTextField = new JTextPane();

		JPanel namePanel = new JPanel();
		namePanel.setLayout(new BorderLayout());
		namePanel.add(new JLabel("Name: "), BorderLayout.WEST);
		nameTextField = new JTextField();
		namePanel.add(nameTextField, BorderLayout.CENTER);

		sequencePanel.add(namePanel, BorderLayout.NORTH);
		sequencePanel.add(sequenceTextField, BorderLayout.CENTER);
		sequencePanel.add(new JLabel("Sequence (5' to 3')"), BorderLayout.NORTH);

		infoPanel = new JPanel();
		infoPanel.setLayout(new BorderLayout());
		infoTextPane = new JTextPane();
		infoTextPane.setEditable(false);
		infoTextPane.setBackground(getBackground());
		scroller = new JScrollPane(infoTextPane);
		scroller.setBackground(getBackground());
		infoPanel.add(scroller);

		cardPanel = new JPanel();
		cardPanel.setLayout(new CardLayout());
		cardPanel.add(sequencePanel, "SEQUENCE_PANEL");
		cardPanel.add(infoPanel, "INFO_PANEL");


		setButton = new JButton("Set");
		setButton.addActionListener(this);


		add(comboBoxPanel, BorderLayout.NORTH);
		add(cardPanel, BorderLayout.CENTER);
		add(setButton, BorderLayout.SOUTH);


		int selectIndex = 0;
		for(BioPart b : bioParts) {
			if(b.getID() == parentBox.getOperation().getInputBioPartID() || b.getID() == parentBox.getOperation().getOutputBioPartID()) {
				comboBox.setSelectedIndex(selectIndex);
				break;
			}
			selectIndex++;
		}

		updateGUI();
	}


	@Override
	public void actionPerformed(ActionEvent e) {
		
		if(e.getSource().equals(comboBox)) {
			updateGUI();
		}
		else if(e.getSource().equals(setButton)) {

/* SETBUTTON ACTIONHANDLER */

			int index = comboBox.getSelectedIndex();


			if(index < bioParts.size()) {
				if(parentBox.getOperation().getType() == Operation.OUTPUT) {
					parentBox.getOperation().setOutputBioPartID(bioParts.get(comboBox.getSelectedIndex()).getID());
				}
				else if(parentBox.getOperation().getType() == Operation.INPUT) {
					parentBox.getOperation().setInputBioPartID(bioParts.get(comboBox.getSelectedIndex()).getID());
				}
			}
			else
			{
				if(parentBox.getOperation().getType() == Operation.OUTPUT) {

					if(sequenceTextField.getText().length() < 1) {
						JOptionPane.showMessageDialog(null,
								"No sequence entered.\nPlease enter a sequence using the letters A, T, C, G and N only.",
							    "Empty sequence.",
							    JOptionPane.ERROR_MESSAGE);
						return;
					}




					if(index < bioParts.size()+1) {  /* Custom Gene with RBS */
						if(!SequenceMaker.checkDNASequence(sequenceTextField.getText())){
							JOptionPane.showMessageDialog(null,
									"Sequence is not valid.\nPlease enter a sequence using the letters A, T, C, G and N only.",
								    "Invalid sequence.",
								    JOptionPane.ERROR_MESSAGE);
							return;
						}

						BioPart newBioPart;
						newBioPart = BioPartsCollection.addPart(BioPart.TYPE_OUTPUT_CUSTOM_GENE_WITH_RBS, "Custom output gene (" + sequenceTextField.getText() +")", "Custom", "n/a", "Custom made", "Custom gene with RBS (" + sequenceTextField.getText() +")", new Sequence(sequenceTextField.getText(), Sequence.FIVE_TO_THREE));
						parentBox.getOperation().setOutputBioPartID(newBioPart.getID());
					}
					else if(index < bioParts.size()+2) { /* Custom Gene without RBS */
						if(!SequenceMaker.checkDNASequence(sequenceTextField.getText())){
							JOptionPane.showMessageDialog(null,
									"Sequence is not valid.\nPlease enter a sequence using the letters A, T, C, G and N only.",
								    "Invalid sequence.",
								    JOptionPane.ERROR_MESSAGE);
							return;
						}

						BioPart newBioPart;
						newBioPart = BioPartsCollection.addPart(BioPart.TYPE_OUTPUT_CUSTOM_RNA_WITHOUT_RBS, "Custom output gene (" + sequenceTextField.getText() +")", "Custom", "n/a", "Custom made", "Custom gene without RBS (" + sequenceTextField.getText() +")", new Sequence(sequenceTextField.getText(), Sequence.FIVE_TO_THREE));
						parentBox.getOperation().setOutputBioPartID(newBioPart.getID());
					}
					else { /* Custom RNA *//* Custom Gene without RBS */
						if(!SequenceMaker.checkRNASequence(sequenceTextField.getText())){
							JOptionPane.showMessageDialog(null,
									"Sequence is not valid.\nPlease enter a sequence using the letters A, U, C, G and N only.",
								    "Invalid sequence.",
								    JOptionPane.ERROR_MESSAGE);
							return;
						}

						BioPart newBioPart;
						newBioPart = BioPartsCollection.addPart(BioPart.TYPE_OUTPUT_CUSTOM_RNA_WITHOUT_RBS, "Custom output RNA (" + sequenceTextField.getText() +")", "Custom", "n/a", "Custom made", "Custom RNA (" + sequenceTextField.getText() +")", new Sequence(sequenceTextField.getText(), Sequence.FIVE_TO_THREE, Sequence.TYPE_RNA));
						parentBox.getOperation().setOutputBioPartID(newBioPart.getID());
					}


				}
			}

			dispose();
		}
	}

	private void updateGUI()
	{
		int index = comboBox.getSelectedIndex();
		if(index < bioParts.size()) {
			BioPart selectedBioPart = bioParts.get(index);
			CardLayout cl = (CardLayout) cardPanel.getLayout();
	        cl.show(cardPanel, "INFO_PANEL");
	        instructionTextPane.setText("\nInformation:");
	        infoTextPane.setText("Name: " + selectedBioPart.getName() + "\n\nBioBrick: " + selectedBioPart.getBioBrickName() + "\n\nSource: " + selectedBioPart.getSource() + "\n\nDescription:\n\n" + selectedBioPart.getDescription());
		}
		else {
			CardLayout cl = (CardLayout) cardPanel.getLayout();
	        cl.show(cardPanel, "SEQUENCE_PANEL");

			if(parentBox.getOperation().getType() == Operation.INPUT) {
				instructionTextPane.setText("Enter the sequence of the RNA that is supposed to be sensed:");
			}
			else if(parentBox.getOperation().getType() == Operation.OUTPUT) {

				if(index < bioParts.size()+1) { /* Custom Gene with RBS */
					instructionTextPane.setText("Enter the sequence of your custom output gene. There will be no ribosome binding site added to it.");
				}
				else if(index < bioParts.size()+2) { /* Custom Gene without RBS */
					instructionTextPane.setText("Enter the sequence of your custom output gene. This programm will automatically add a ribosome binding site upstream of your sequence.");
				}
				else { /* Custom RNA */
					instructionTextPane.setText("Enter the sequence of your custom output RNA.");
				}
			}
		}
		infoTextPane.setCaretPosition(0);
	}


}